/**
 * \addtogroup Cflash
 * @{
 */

/*****************************************************************************/
/**
 * \file Cflash.c
 * Sector level Compact Flash routines for the 8051 and Keil C51.
 * \author Copyright (c) 2005, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.1
 * \date 21-02-05
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

#include "CFlash.h" 

/*****************************************************************************
 *
 *                          Global Variables Definitions  
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *                          Private Function Prototypes   
 *
 *****************************************************************************/
unsigned char cflash_rd_reg(unsigned char reg);
void cflash_wr_reg(unsigned char reg, unsigned char byte);


/*****************************************************************************
 *
 *                        Public Function Implementation  
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                                cflash_init()  
 *
 *****************************************************************************/
void cflash_init(void)
{
    // Set Data outputs as inputs
    CF_DATA_PORT = 0xFF;

    // These are active low signals
    WE  = 1;
    OE  = 1;
    CE1 = 1;
}

/*****************************************************************************
 *
 *                             cflash_rd_sector()
 *
 *****************************************************************************/
void cflash_rd_sector(unsigned long sector, unsigned char xdata * buffer)
{
    unsigned int i;

    // Load sector count register with request for 1 sector
    cflash_wr_reg(CF_SEC_CNT_REG, 0x01);

    // Load sector number register with LBA 7..0
    cflash_wr_reg(CF_SEC_NUM_REG, sector);

    // Load cylinder low register with LBA 15..8
    cflash_wr_reg(CF_CYL_LO_REG, sector >> 8);

    // Load cylinder high register with LBA 23..16
    cflash_wr_reg(CF_CYL_HI_REG, sector >> 16);

    // Load C/D/H register with LBA 27..24, drive num & LBA addressing request 
    cflash_wr_reg(CF_HEAD_REG, CF_LBA | sector >> 24);

    // Issue read sector request
    cflash_wr_reg(CF_COMMAND_REG, CF_READ_SEC);

    // Copy 512 byte sector into local buffer
    for(i = 0; i < CF_SECTOR_SIZE; i++)
    {
        buffer[i] = cflash_rd_reg(CF_DATA_REG);
    }

}

/*****************************************************************************
 *
 *                             cflash_wr_sector()
 *
 *****************************************************************************/
void cflash_wr_sector(unsigned long sector, unsigned char xdata * buffer)
{
    unsigned int i;

    // Load sector count register with request for 1 sector
    cflash_wr_reg(CF_SEC_CNT_REG, 0x01);

    // Load sector number register with LBA 7..0
    cflash_wr_reg(CF_SEC_NUM_REG, sector);

    // Load cylinder low register with LBA 15..8
    cflash_wr_reg(CF_CYL_LO_REG, sector >> 8);

    // Load cylinder high register with LBA 23..16
    cflash_wr_reg(CF_CYL_HI_REG, sector >> 16);

    // Load C/D/H register with LBA 27..24, drive num & LBA addressing request 
    cflash_wr_reg(CF_HEAD_REG, CF_LBA | sector >> 24);

    // Issue read sector request
    cflash_wr_reg(CF_COMMAND_REG, CF_WRITE_SEC);

    // Copy local buffer into 512 byte sector 
    for(i = 0; i < CF_SECTOR_SIZE; i++)
    {
         cflash_wr_reg(CF_DATA_REG, buffer[i]);
    }
}


/*****************************************************************************
 *
 *                        Private Function Implementation
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                              cflash_wr_reg()          
 *
 *****************************************************************************/
void cflash_wr_reg(unsigned char reg, unsigned char byte)
{
    // Select Register
    CF_ADDRRESS_PORT &= 0xF8;         // Clear A0..A2 on Bits 0..2
    CF_ADDRRESS_PORT |= reg & 0x07;   // Set register address on A0..A2

    // Output Data
    CF_DATA_PORT = byte;

    // Wait until Compact Flash Ready
    while (~RDY);

    // Select Compact Flash Chip
    CE1 = 0;

    // Select Write operation
    WE = 0;

    // Wait 150ns

    // Clear Write operation
    WE = 1;

    // Wait 30ns

    // Deselect Compact Flash Chip
    CE1 = 1;

    // Set Data outputs as inputs
    CF_DATA_PORT = 0xFF;
}    

/*****************************************************************************
 *
 *                              cflash_rd_reg()          
 *
 *****************************************************************************/
unsigned char cflash_rd_reg(unsigned char reg)
{
    unsigned char byte = 0;

    // Select Register
    CF_ADDRRESS_PORT &= 0xF8;         // Clear A0..A2 on Bits 0..2
    CF_ADDRRESS_PORT |= reg & 0x07;   // Set register address on A0..A2

    // Wait until Compact Flash Ready
    while (~RDY);

    // Select Compact Flash Chip
    CE1 = 0;

    // Select Read operation
    OE = 0;

    // Wait 125ns

    // Input Data
    byte = CF_DATA_PORT;

    // Clear Read operation
    OE = 1;

    // Wait 20ns

    // Deselect Compact Flash Chip
    CE1 = 1;

    return byte;
}

/** @} */


